@@ -29,22 +29,26 @@
 
 import pathlib
 import re
+import json
 
 from synth_forc.cli.response import Response
+from synth_forc.cli.response import ResponseStatusEnum
 from synth_forc.qt.main_window import Ui_MainWindow
 
-from PyQt6 import QtGui
+from PyQt6 import QtGui, QtCore
 
 from PyQt6.QtGui import QPixmap
 from PyQt6.QtGui import QRegularExpressionValidator
 
-from PyQt6.QtCore import QCoreApplication, QRunnable, QThreadPool, pyqtSlot
+from PyQt6.QtCore import QCoreApplication, QRunnable, QThreadPool, pyqtSlot, QRectF
 from PyQt6.QtCore import QRegularExpression
+from PyQt6.QtCore import Qt
 
 from PyQt6.QtWidgets import QMainWindow
 from PyQt6.QtWidgets import QGraphicsScene
 from PyQt6.QtWidgets import QMessageBox
 from PyQt6.QtWidgets import QFileDialog
+from PyQt6.QtWidgets import QGraphicsPixmapItem
 
 from PIL import Image, ImageQt
 
@@ -58,6 +62,8 @@ from synth_forc.gui.spinner import QtWaitingSpinner
 
 from synth_forc.plotter_threads import GenerateForcImages
 
+from synth_forc.logger import get_logger
+
 
 class MainWindow(QMainWindow, Ui_MainWindow):
 
@@ -66,52 +72,116 @@ class MainWindow(QMainWindow, Ui_MainWindow):
 
     def __init__(self, temp_dir):
 
+        self.logger = get_logger()
+
         QMainWindow.__init__(self)
+        self.logger.debug("'QMainWindow' is initialized.")
+
         Ui_MainWindow.__init__(self)
+        self.logger.debug("'Ui_MainWindow' is initialized.")
 
         self.setupUi(self)
-
+        self.logger.debug("User interface has been set up.")
 
         self.temp_dir = temp_dir
-        self.temp_dir_space_manager = TemporaryDirectorySpaceManager(self.temp_dir)
-        self.settings = Settings.get_settings()
         self.db_file = None
         self.synthforc_db = None
         self.forc_save_file = None
         self.forc_loops_save_file = None
         self.size_distr_save_file = None
         self.aratio_distr_save_file = None
+        self.logger.debug("Local variables have been initialized.")
+
+        self.temp_dir_space_manager = TemporaryDirectorySpaceManager(self.temp_dir)
+        self.logger.debug(f"'self.temp_dir_space_manager' has been created with self.temp_dir: {self.temp_dir}.")
+
+        self.settings = Settings.get_settings()
+        self.logger.debug("'self.settings' has been created.")
 
         self.btn_generate.clicked.connect(self.btn_generate_action)
+        self.logger.debug("'self.btn_generate.clicked' event has been connected to action 'self.btn_generate_action'")
 
         self.txt_size_distr_shape.textChanged.connect(self.txt_size_distr_shape_change_action)
+        self.logger.debug("'self.txt_size_distr_shape.textChanged' event has been connected to action 'self.txt_size_distr_shape_change_action'")
+
         self.txt_size_distr_location.textChanged.connect(self.txt_size_distr_location_change_action)
+        self.logger.debug("'self.txt_size_distr_location.textChanged' event has been connected to action 'self.txt_size_distr_location_change_action'")
+
         self.txt_size_distr_scale.textChanged.connect(self.txt_size_distr_scale_change_action)
+        self.logger.debug("'self.txt_size_distr_scale.textChanged' event has been connected to action 'self.txt_size_distr_scale_change_action'")
 
         self.txt_aratio_distr_shape.textChanged.connect(self.txt_aratio_distr_shape_change_action)
+        self.logger.debug("'self.txt_aratio_distr_shape.textChanged' event has been connected to action 'self.txt_aratio_distr_shape_change_action'")
+
         self.txt_aratio_distr_location.textChanged.connect(self.txt_aratio_distr_location_change_action)
+        self.logger.debug("'self.txt_aratio_distr_location.textChanged' event has been connected to action 'self.txt_aratio_distr_location_change_action'")
+
         self.txt_aratio_distr_scale.textChanged.connect(self.txt_aratio_scale_change_action)
+        self.logger.debug("'self.txt_aratio_distr_scale.textChanged' event has been connected to action 'self.txt_aratio_scale_change_action'")
 
         self.menu_configure.triggered.connect(self.menu_configure_action)
+        self.logger.debug("'self.menu_configure.triggered' event has been connected to action 'self.menu_configure_action'")
+
         self.menu_close.triggered.connect(self.menu_close_action)
+        self.logger.debug("'self.menu_close.triggered' event has been connected to action 'self.menu_close_action'")
+
         self.menu_save.triggered.connect(self.menu_save_action)
+        self.logger.debug("'self.menu_save.triggered' event has been connected to action 'self.menu_save_action'")
+
         self.menu_load.triggered.connect(self.menu_load_action)
+        self.logger.debug("'self.menu_load.triggered' event has been connected to action 'self.menu_load_action'")
+
+        self.logger.debug(f"self.graphics_size_distribution size: "
+                          f"{self.graphics_size_distribution.sceneRect().width()}, "
+                          f"{self.graphics_size_distribution.sceneRect().height()}")
 
         self.size_distr_scene = QGraphicsScene(self)
+        self.logger.debug(f"Created self.size_distr_scene: {self.size_distr_scene}")
+
         self.graphics_size_distribution.setScene(self.size_distr_scene)
+        self.logger.debug(f"Set graphics_size_distribution scene")
+
         self.size_distr_pixmap = self.size_distr_scene.addPixmap(QPixmap())
+        self.logger.debug(f"Created self.size_distr_pixmap: {self.size_distr_pixmap}")
+
+        self.logger.debug(f"self.graphics_aratio_distribution size: "
+                          f"{self.graphics_aratio_distribution.sceneRect().width()}, "
+                          f"{self.graphics_aratio_distribution.sceneRect().height()}")
 
         self.aratio_distr_scene = QGraphicsScene(self)
+        self.logger.debug(f"Created self.aratio_distr_scene: {self.aratio_distr_scene}")
+
         self.graphics_aratio_distribution.setScene(self.aratio_distr_scene)
+        self.logger.debug(f"Set graphics_aratio_distribution scene")
+
         self.aratio_distr_pixmap = self.aratio_distr_scene.addPixmap(QPixmap())
+        self.logger.debug(f"Created self.aratio_distr_pixmap: {self.aratio_distr_pixmap}")
+
+        self.logger.debug(f"self.graphics_forcs size: "
+                          f"{self.graphics_forcs.sceneRect().width()}, "
+                          f"{self.graphics_forcs.sceneRect().height()}")
 
         self.forc_scene = QGraphicsScene(self)
+        self.logger.debug(f"Created self.forc_scene: {self.forc_scene}")
+
         self.graphics_forcs.setScene(self.forc_scene)
+        self.logger.debug(f"Set graphics_forcs scene")
+
         self.forc_pixmap = self.forc_scene.addPixmap(QPixmap())
+        self.logger.debug(f"Created self.forc_pixmap: {self.forc_pixmap}")
+
+        self.logger.debug(f"self.graphics_loops size: "
+                          f"{self.graphics_loops.sceneRect().width()}, "
+                          f"{self.graphics_loops.sceneRect().height()}")
 
         self.forc_loops_scene = QGraphicsScene(self)
+        self.logger.debug(f"Created self.forc_loops_scene: {self.forc_loops_scene}")
+
         self.graphics_loops.setScene(self.forc_loops_scene)
+        self.logger.debug(f"Set self.graphics_loops scene")
+
         self.forc_loops_pixmap = self.forc_loops_scene.addPixmap(QPixmap())
+        self.logger.debug(f"Created self.forc_loops_pixmap: {self.forc_loops_pixmap}")
 
         self.txt_size_distr_shape.setValidator(QRegularExpressionValidator(MainWindow.re_float))
         self.txt_size_distr_location.setValidator(QRegularExpressionValidator(MainWindow.re_float))
@@ -127,27 +197,28 @@ class MainWindow(QMainWindow, Ui_MainWindow):
 
         self.initialise_distribution_plots()
 
-    def set_forc_image(self, image, no_adjust=False):
+    def set_forc_image(self, image_png_file, no_adjust=False):
+        logger = self.logger
 
         # FORC plot
-        forc_image = Image.open(image)
+        forc_image = Image.open(image_png_file)
         forc_image_width, forc_image_height = forc_image.size
         if not no_adjust:
             forc_image = forc_image.crop((1200, 0, forc_image_width - 150, forc_image_height))
+        width, height = forc_image.size
         forc_qimage = ImageQt.ImageQt(forc_image)
+
         self.forc_pixmap.setPixmap(QtGui.QPixmap.fromImage(forc_qimage))
-        self.graphics_forcs.resetTransform()
-        self.graphics_forcs.scale(0.15, 0.15)
+        self.graphics_forcs.setSceneRect(0, 0, width, height)
 
-    def set_forc_loops_image(self, image):
+    def set_forc_loops_image(self, image_png_file):
 
-        # FORC loops plot
-        forc_loops_image = Image.open(image)
-        # forc_loops_width, forc_loops_height = forc_loops_image.size
-        forc_loops_qimage = ImageQt.ImageQt(forc_loops_image)
-        self.forc_loops_pixmap.setPixmap(QtGui.QPixmap.fromImage(forc_loops_qimage))
-        self.graphics_loops.resetTransform()
-        self.graphics_loops.scale(0.15, 0.15)
+        image = Image.open(image_png_file)
+        width, height = image.size
+        qimage = ImageQt.ImageQt(image)
+
+        self.forc_loops_pixmap.setPixmap(QtGui.QPixmap.fromImage(qimage))
+        self.graphics_loops.setSceneRect(0, 0, width, height)
 
     def btn_generate_action(self):
 
@@ -158,55 +229,42 @@ class MainWindow(QMainWindow, Ui_MainWindow):
                         "Please load a synth-FORC file using the 'File' menu")
             dlg.exec()
         else:
-            smoothing_factor = int(self.txt_smoothing_factor.text())
-            ar_shape = float(self.txt_aratio_distr_shape.text())
-            ar_location = float(self.txt_aratio_distr_location.text())
-            ar_scale = float(self.txt_aratio_distr_scale.text())
-            size_shape = float(self.txt_size_distr_shape.text())
-            size_location = float(self.txt_size_distr_location.text())
-            size_scale = float(self.txt_size_distr_scale.text())
-
             pool = QThreadPool.globalInstance()
 
             self.spinner.start()
 
             pool.start(
                 GenerateForcImages(
-                    1, self.db_file, ar_shape, ar_location, ar_scale, size_shape, size_location, size_scale,
-                    self.temp_dir_space_manager.forc_plot_png,
-                    self.temp_dir_space_manager.forc_plot_pdf,
-                    self.temp_dir_space_manager.forc_plot_jpg,
-                    self.temp_dir_space_manager.forc_loops_plot_png,
-                    self.temp_dir_space_manager.forc_loops_plot_pdf,
-                    self.temp_dir_space_manager.forc_loops_plot_jpg,
-                    smoothing_factor, self.settings.dpi, self
+                    1, self, self.logger
                 )
             )
 
     @pyqtSlot(str, str)
     def forc_generation_complete(self, stdout: str, stderr: str):
+        logger = self.logger
 
         self.spinner.stop()
-        print(f"stdout: {stdout}")
-        print(f"stderr: {stderr}")
 
-        response = Response(json=stdout)
+        logger.debug(f"stdout: {stdout}")
+        logger.debug(f"stderr: {stderr}")
 
-        if response.status == Response.Status.EMPTY_LOOPS:
+        response = Response(json.loads(stdout))
+
+        if response.status == ResponseStatusEnum.EMPTY_LOOPS.value:
             print("Empty loops")
-        elif response.status == Response.Status.SUCCESS:
+        elif response.status == ResponseStatusEnum.SUCCESS.value:
 
             # Display the images.
             self.set_forc_image(response.forc_png)
             self.set_forc_loops_image(response.forc_loop_png)
 
-        elif response.status == Response.Status.EMPTY_BINS:
+        elif response.status == ResponseStatusEnum.EMPTY_BINS.value:
 
             # Display empty images
             self.set_forc_image(self.temp_dir_space_manager.empty_image_png, no_adjust=True)
             self.set_forc_loops_image(self.temp_dir_space_manager.empty_image_png)
 
-        elif response.status == Response.Status.EXCEPTION:
+        elif response.status == ResponseStatusEnum.EXCEPTION.value:
             print(response.exception)
         else:
             print("Unknown error occurred")
@@ -234,7 +292,16 @@ class MainWindow(QMainWindow, Ui_MainWindow):
             self.update_size_distribution_plot()
             self.update_aratio_distribution_plot()
 
-            print(self.synthforc_db.aratios)
+        # Set the forc and forc loop to zero
+        image = Image.open(self.temp_dir_space_manager.empty_image_png)
+        width, height = image.size
+        qimage = ImageQt.ImageQt(image)
+        self.forc_pixmap.setPixmap(QtGui.QPixmap.fromImage(qimage))
+        self.graphics_forcs.setSceneRect(0, 0, width, height)
+        self.forc_loops_pixmap.setPixmap(QtGui.QPixmap.fromImage(qimage))
+        self.graphics_loops.setSceneRect(0, 0, width, height)
+
+            #print(self.synthforc_db.aratios)
 
     def txt_size_distr_shape_change_action(self, text):
         self.update_size_distribution_plot()
@@ -294,13 +361,20 @@ class MainWindow(QMainWindow, Ui_MainWindow):
         self.update_size_distribution_plot()
         self.update_aratio_distribution_plot()
 
-    def set_size_image(self, image):
-        image = Image.open(image)
+        image = Image.open(self.temp_dir_space_manager.empty_image_png)
+        width, height = image.size
+        qimage = ImageQt.ImageQt(image)
+        self.forc_pixmap.setPixmap(QtGui.QPixmap.fromImage(qimage))
+        self.graphics_forcs.setSceneRect(0, 0, width, height)
+        self.forc_loops_pixmap.setPixmap(QtGui.QPixmap.fromImage(qimage))
+        self.graphics_loops.setSceneRect(0, 0, width, height)
+
+    def set_size_image(self, image_png_file):
+        image = Image.open(image_png_file)
         width, height = image.size
         qimage = ImageQt.ImageQt(image)
         self.size_distr_pixmap.setPixmap(QtGui.QPixmap.fromImage(qimage))
-        self.graphics_size_distribution.resetTransform()
-        self.graphics_size_distribution.scale(0.15, 0.15)
+        self.graphics_size_distribution.setSceneRect(0, 0, width, height)
 
     def update_size_distribution_plot(self):
 
@@ -328,13 +402,12 @@ class MainWindow(QMainWindow, Ui_MainWindow):
         if self.temp_dir_space_manager.size_distribution_plot_png is not None:
             self.set_size_image(self.temp_dir_space_manager.size_distribution_plot_png)
 
-    def set_aratio_image(self, image):
-        image = Image.open(image)
+    def set_aratio_image(self, image_png_file):
+        image = Image.open(image_png_file)
         width, height = image.size
         qimage = ImageQt.ImageQt(image)
         self.aratio_distr_pixmap.setPixmap(QtGui.QPixmap.fromImage(qimage))
-        self.graphics_aratio_distribution.resetTransform()
-        self.graphics_aratio_distribution.scale(0.15, 0.15)
+        self.graphics_aratio_distribution.setSceneRect(0, 0, width, height)
 
     def update_aratio_distribution_plot(self):
 
@@ -353,8 +426,10 @@ class MainWindow(QMainWindow, Ui_MainWindow):
         # If there is a BinsEmptyException then don't display.
         try:
             self.temp_dir_space_manager.create_aratio_distribution_plot(
-                self.get_aratio_distr_shape(), self.get_aratio_distr_location(), self.get_aratio_distr_scale(), aratio_bins
-            )
+                self.get_aratio_distr_shape(),
+                self.get_aratio_distr_location(),
+                self.get_aratio_distr_scale(),
+                aratio_bins)
         except BinsEmptyException as e:
             self.set_aratio_image(self.temp_dir_space_manager.empty_image_png)
             return
@@ -395,3 +470,45 @@ class MainWindow(QMainWindow, Ui_MainWindow):
             self.aratio_distr_save_file = dlg.txt_aspect_ratio_distribution.text()
         else:
             self.aratio_distr_save_file = None
+
+    def showEvent(self, a0: QtGui.QShowEvent) -> None:
+        logger = self.logger
+
+        logger.debug(f"self.graphics_aratio_distribution size: "
+                     f"{self.graphics_aratio_distribution.sceneRect().width()}, "
+                     f"{self.graphics_aratio_distribution.sceneRect().height()}")
+
+    def resizeEvent(self, event: QtGui.QResizeEvent) -> None:
+        super().resizeEvent(event)
+
+        logger = self.logger
+        logger.debug(f"Resizing window.")
+
+        new_size = event.size()
+        logger.debug(f"new_size: {new_size}")
+
+        aratio_size = self.aratio_distr_scene.sceneRect().size()
+        logger.debug(f"aratio_size: {aratio_size}")
+
+        aratio_x_scale_factor = new_size.width() / (aratio_size.width()*2.7)
+        aratio_y_scale_factor = new_size.height() / (aratio_size.height()*2.7)
+
+        self.graphics_aratio_distribution.resetTransform()
+        self.graphics_aratio_distribution.setTransform(
+            self.graphics_aratio_distribution.transform().scale(aratio_x_scale_factor,
+                                                                aratio_x_scale_factor))
+
+        self.graphics_size_distribution.resetTransform()
+        self.graphics_size_distribution.setTransform(
+            self.graphics_size_distribution.transform().scale(aratio_x_scale_factor,
+                                                              aratio_x_scale_factor))
+
+        self.graphics_forcs.resetTransform()
+        self.graphics_forcs.setTransform(
+            self.graphics_forcs.transform().scale(aratio_x_scale_factor,
+                                                  aratio_x_scale_factor))
+
+        self.graphics_loops.resetTransform()
+        self.graphics_loops.setTransform(
+            self.graphics_loops.transform().scale(aratio_x_scale_factor,
+                                                  aratio_x_scale_factor))
